# 基于eBPF的Raft协议加速

## 项目简介

### 项目名称
Project266: 基于eBPF的Raft协议加速

### 项目背景
Raft协议是当前国际上广泛采用的分布式共识算法，主要应用于etcd等分布式存储系统中实现数据复制。然而，目前Raft协议在用户态实现，执行过程中会引发频繁的用户态和核心态切换，并且需要经过内核网络协议栈，导致较大的性能开销。

Dragonboat是一个高性能的纯Go语言实现的多组Raft共识算法库。Raft共识算法使得系统在多数成员在线的情况下依然可以持续运行。例如，在一个拥有5台服务器的Raft集群中，即使有两台服务器故障，系统仍能继续工作。Raft协议向客户端展现一个单机节点，始终提供强一致性的保证。同时，所有在线的成员节点都可以提供读操作，从而提高读吞吐量。Dragonboat承担了所有Raft相关的技术难点，使用户只需关注各自的应用领域。

eBPF（extended Berkeley Packet Filter）是Linux内核的新特性，允许用户在内核中安全地运行自定义功能。eBPF支持在内核态下执行用户定义的字节码，可以用来实现高性能的网络数据包处理、性能监控、安全检查等功能。通过在内核态实现Raft协议中的关键流程，可以减少用户态与核心态之间的频繁切换，从而显著提高协议的性能。

Electrode是一个通过eBPF加速分布式协议的项目，它利用eBPF技术优化Paxos算法的性能。Electrode的研究表明，通过在内核中执行部分协议逻辑，可以显著减少协议运行的延迟和资源消耗。我们计划基于Electrode的思路，优化Dragonboat的Raft协议实现，将其消息转发和接收流程卸载到内核中运行，从而提升性能。

### 项目目标
本项目旨在基于Raft协议的开源实现Dragonboat，通过eBPF程序将Raft协议的消息转发和接收流程卸载到内核中运行，从而提升Raft协议的性能。

### 项目特征
- 掌握Dragonboat的相关代码
- 掌握eBPF的基本原理和编程方法
- 掌握Linux内核网络协议栈报文收发相关代码

## 团队介绍
- 队名: 三只逸仙鸭鸭win
- 学校: 中山大学
- 指导老师：陈鹏飞，陈亮
- 队员: 金鹏飞，赵文宽，蔡燊林

## 项目目录

```
/ (根目录)
├── README.md                # 项目概述
├── LICENSE                  # 项目许可证
├── docs(######adding!!!)                     # 文档目录
│   ├── 项目介绍.md          # 项目介绍、背景及目标
│   ├── 设计思路.md          # 设计思路和整体架构
│   ├── 实现描述.md          # 详细的实现描述和步骤
│   ├── 遇到的问题和解决方案.md # 在项目过程中遇到的问题及解决方案
│   ├── 参考资料.md          # 参考文献及资料列表
│   ├── 会议记录和讨论.md    # 项目过程中的会议记录和讨论要点
│   ├── eBPF学习.md          # eBPF学习笔记和代码示例
│   ├── Raft协议学习.md      # Raft协议学习笔记和论文解读
│   ├── dragonboat_study    # Dragonboat代码阅读和分析
│   └── 计算机网络学习.md    # 计算机网络、TCP/UDP相关知识学习笔记
├── src                      # 源代码目录
│   ├── dragonboat_mod       # 对Dragonboat的修改代码
│   ├── eBPF_programs        # eBPF程序
│   └── test                 # 测试代码和测试结果
├── results                  # 项目结果
│   ├── 测试结果分析.md       # 功能/性能/创新性分析
│   ├── 对比分析.md          # 与类似项目的对比分析
│   └── 其他结果文件
└── tools                    # 辅助工具和脚本
    ├── scripts              # 各种脚本
    └── configs              # 配置文件
```

## 参考资料（这里可以增加...）
- Zhou, Yang et al. “Electrode: Accelerating Distributed Protocols with eBPF”. Symposium on Networked Systems Design and Implementation (2023).
- Dragonboat: https://github.com/lni/dragonboat
- eBPF: https://ebpf.io/

## 许可证
本项目采用 GPL-3.0 License 许可证。

## 联系我们
- 邮箱: wenkuanzhao418@gmail.com
- Github: https://github.com/MarshmallowLight

